


Public Class TempoChangeBuilder
    Implements IMessageBuilder

    ' -----------------------------------------  for shifting bits for packing and unpacking tempo values
    Private Const Shift As Integer = 8

    Private m_tempo As Integer = PpqnClock.DefaultTempo
    Private m_result As MetaMessage = Nothing
    Private changed As Boolean = True


    Public Sub New()
    End Sub

    Public Sub New(ByVal e As MetaMessage)
        Initialize(e)
    End Sub

    Public Sub Initialize(ByVal e As MetaMessage)
        If e Is Nothing Then
            Throw New ArgumentNullException("e")
        ElseIf e.MetaType <> MetaType.Tempo Then
            Throw New ArgumentException("Wrong meta message type.", "e")
        End If
        Dim t As Integer = 0
        ' -------------------------------------------------- if this platform uses little endian byte order
        If BitConverter.IsLittleEndian Then
            Dim d As Integer = e.Length - 1
            ' ---------------------------------------------- pack tempo
            For i As Integer = 0 To e.Length - 1
                t = t Or (CInt(e(d)) << (Shift * i))
                d -= 1
            Next
            ' ---------------------------------------------- else this platform uses big endian byte order
        Else
            ' ---------------------------------------------- pack tempo
            For i As Integer = 0 To e.Length - 1
                t = t Or (CInt(e(i)) << (Shift * i))
            Next
        End If
        m_tempo = t
    End Sub

    Public Property Tempo() As Integer
        Get
            Return m_tempo
        End Get
        Set(ByVal value As Integer)
            If value < 0 Then
                Throw New ArgumentOutOfRangeException("Tempo", value, "Tempo is out of range.")
            End If
            m_tempo = value
            changed = True
        End Set
    End Property

    Public ReadOnly Property Result() As MetaMessage
        Get
            Return m_result
        End Get
    End Property

    Public Sub Build() Implements IMessageBuilder.Build
        ' -------------------------------------- if the tempo is changed since the last time the message was built
        If changed Then
            Dim data As Byte() = New Byte(MetaMessage.TempoLength - 1) {}
            '
            ' ----------------------------------------------- if this platform uses little endian byte order
            If BitConverter.IsLittleEndian Then
                Dim d As Integer = data.Length - 1
                ' ------------------------------------------- unpack tempo
                For i As Integer = 0 To data.Length - 1
                    data(d) = CByte((m_tempo >> (Shift * i)))
                    d -= 1
                Next
                ' ------------------------------------------ else this platform uses big endian byte order
            Else
                ' ------------------------------------------ unpack tempo
                For i As Integer = 0 To data.Length - 1
                    data(i) = CByte((m_tempo >> (Shift * i)))
                Next
            End If
            changed = False
            m_result = New MetaMessage(MetaType.Tempo, data)
        End If
    End Sub

End Class
